//
//  CameraNativeView.swift
//  Pods
//
//  Created by Sarbjyot Chahal on 31.03.22.
//

import Foundation
import Flutter
import ScanbotSDK
class CameraNativeView : NSObject, FlutterPlatformView, SBSDKBarcodeScannerViewControllerDelegate, CallErrorHandler {
    
    private let kViewControllerAppearDelay = 0.2
    private var rootView: UIView
    private var cameraContainerView: UIView
    private var shouldDetectBarcodes: Bool = true
    private var waitingForImageSnap: Bool = false
    private var viewController: SBSDKBarcodeScannerViewController?
    private var cameraMethodChannel = SwiftScanbotSdkPlugin.cameraMethodChannel
    private let detectorChannel = SwiftScanbotSdkPlugin.classicalCameraDetectorChannel
    private let configuration : [String: Any?]?
    private var imageGenerationType: SBSDKBarcodeImageGenerationType = SBSDKBarcodeImageGenerationType.none
    private var cachedCodes: [SBSDKBarcodeScannerResult]? = nil
    
    init(
        frame: CGRect,
        viewIdentifier viewId: Int64,
        arguments args: Any?,
        binaryMessenger messenger: FlutterBinaryMessenger?
    ) {
        rootView = UIView()
        cameraContainerView = UIView()
        configuration = args as? [String:Any?]
        super.init()
        
        detectorChannel?.setMethodCallHandler({
            (call: FlutterMethodCall, result: @escaping FlutterResult) -> Void in
            self.handleDetectorCall(call: call, result: result)
        })
        
        cameraMethodChannel?.setMethodCallHandler({
            (call: FlutterMethodCall, result: @escaping FlutterResult) -> Void in
            self.handleCameraCall(call: call, result: result)
        })
        
        DispatchQueue.main.asyncAfter(deadline: .now() + kViewControllerAppearDelay) { [weak self] in
            guard let _containerView = self?.rootView else {
                return
            }
            self?.createNativeView(view: _containerView)
        }
    }
    
    func view() -> UIView {
        return rootView
    }
    
    fileprivate func sendLicenseErrorToDetector(_ callHandler: FlutterMethodChannel) {
        let isValid = ScanbotSDK.isLicenseValid()
        let status = ScanbotSDK.licenseStatus()
        let expirationDate = ScanbotSDK.licenseExpirationDate()
        
        var dictionary: [String: Any?] = ["isLicenseValid": isValid, "status": status.stringName]
        if let date = expirationDate {
            let formatter = DateFormatter()
            formatter.timeZone = TimeZone(abbreviation: "UTC")
            formatter.dateFormat = "YYYY-MM-dd HH:mm:ss"
            formatter.locale = Locale(identifier: "en_EN")
            let dateString = formatter.string(from: date)
            dictionary["expirationDate"] = dateString
        }
        dictionary["message"] = "license is no more valid for this feature"
        callHandler.invokeMethod("onLicenseError", arguments: dictionary.encodedString)
    }
    
    func createNativeView(view _view: UIView){
        self.rootView.frame = CGRect(x: 0, y: 0, width: UIScreen.main.bounds.width, height: UIScreen.main.bounds.height)
        self.cameraContainerView.translatesAutoresizingMaskIntoConstraints = false
        self.cameraContainerView.frame = CGRect(x: 0, y: 0, width: UIScreen.main.bounds.width, height: UIScreen.main.bounds.height)
        
        DispatchQueue.main.async { [weak self] in
            guard let cameraContainerView = self?.cameraContainerView else {
                fatalError("Cannot initialize CameraNativeView: containerView is NULL")
            }
            _view.addSubview(cameraContainerView)
            NSLayoutConstraint.activate([
                cameraContainerView.topAnchor.constraint(equalTo: _view.topAnchor),
                cameraContainerView.bottomAnchor.constraint(equalTo: _view.bottomAnchor),
                cameraContainerView.leadingAnchor.constraint(equalTo: _view.leadingAnchor),
                cameraContainerView.trailingAnchor.constraint(equalTo: _view.trailingAnchor),
            ])
            guard let rootViewController = UIApplication.shared.keyWindow?.rootViewController else {
                fatalError("Cannot create native view: application keyWindow is NULL")
            }
            
            self?.viewController = SBSDKBarcodeScannerViewController(parentViewController: rootViewController,
                                                                     parentView: cameraContainerView)
            self?.viewController?.delegate = self
            self?.cameraMethodChannel?.invokeMethod(MethodChannelConstants.kOnCameraOpenMethod, arguments:"")
            self?.setBarcodeScannerConfiguration(configuration:self?.configuration)
        }
        if(!ScanbotSDK.isLicenseValid()){
            DispatchQueue.main.async { [weak self] in
                guard let callHandler = self?.detectorChannel else {
                    fatalError("callHandler cannot be nil")
                }
                self?.sendLicenseErrorToDetector(callHandler)
            }
        }
        
    }
    
    func dispose(){
        viewController?.dismiss(animated: true)
        viewController = nil
    }
    
    func barcodeScannerController(_ controller: SBSDKBarcodeScannerViewController, didCaptureBarcodeImage barcodeImage: UIImage) {
        // to snap picture
        
        cameraMethodChannel?.invokeMethod("onShowHeavyProcessing", arguments: true)
        
        guard let uuid = SBSDKUIPageFileStorage.default().add(barcodeImage) else { return  }
        let uri = SBSDKUIPageFileStorage.default().imageURL(withPageFileID: uuid, pageFileType: .original)
        
        guard let codes = cachedCodes else { return }
        sendDetectedBarcodesResult(codes, uri?.standardizedFileURL.absoluteString ?? "")
        
        cameraMethodChannel?.invokeMethod("onShowHeavyProcessing", arguments: false)
        waitingForImageSnap = false
    }
    
    func barcodeScannerControllerShouldDetectBarcodes(_ controller: SBSDKBarcodeScannerViewController) -> Bool {
        return shouldDetectBarcodes && !waitingForImageSnap
    }
    
    func barcodeScannerController(_ controller: SBSDKBarcodeScannerViewController, didChangeViewFinderRect rect: CGRect) {
        let dictionary: [String: Any?] = ["left": rect.minX,
                                          "top": rect.minY,
                                          "right": rect.maxX,
                                          "bottom": rect.maxY,]
        
        SwiftScanbotSdkPlugin.cameraFinderChannel?.invokeMethod("finderSizeChanged", arguments: dictionary)
        
    }
    
    fileprivate func sendDetectedBarcodesResult(_ codes: [SBSDKBarcodeScannerResult],_ barcodeImageURI: String?) {
        let resultDict: [String: Any?] = ["barcodeItems": codes.map({ $0.encodedDictionary }), "barcodeImageURI" : barcodeImageURI]
        
        DispatchQueue.main.async { [weak self] in
            guard let callHandler = self?.detectorChannel else {
                fatalError("callHandler cannot be nil")
            }
            callHandler.invokeMethod("onBarcodeDetected", arguments: resultDict.encodedString)
        }
    }
    
    func barcodeScannerController(_ controller: SBSDKBarcodeScannerViewController, didDetectBarcodes codes: [SBSDKBarcodeScannerResult]) {
        if(!codes.isEmpty){
            switch(imageGenerationType){
            case .none:
                sendDetectedBarcodesResult(codes, nil)
            case .fromVideoFrame:
                waitingForImageSnap = true
                cachedCodes = codes
            case .capturedImage:
                waitingForImageSnap = true
                cachedCodes = codes
            @unknown default:
                sendDetectedBarcodesResult(codes, nil)
            }
        }
    }
    
    
    func setBarcodeScannerConfiguration(configuration:[String:Any?]?) {
        if let flashEnabled = configuration?["flashEnabled"] as? Bool {
            viewController?.flashLightEnabled = flashEnabled
        }
        
        if let cameraZoomFactor = configuration?["cameraZoomFactor"] as? CGFloat {
            viewController?.cameraZoomFactor = cameraZoomFactor
        }
        
        if let finderConfiguration: [String:Any] = configuration?["finder"] as? [String : Any] {
            viewController?.shouldUseFinderFrame = true
            viewController?.viewFinderLineColor = UIColor(hexString: "#00000000")
            viewController?.viewFinderBackgroundColor = UIColor(hexString: "#00000000")
            
            if let finderMinimumPadding = finderConfiguration["finderMinimumPadding"] as? Double {
                viewController?.viewFinderLineWidth = finderMinimumPadding
            }
            
            if
                let finderAspectRatio = finderConfiguration["finderAspectRatio"] as? [String:Any],
                let width = finderAspectRatio["width"] as? Double,
                let height = finderAspectRatio["height"] as? Double
            {
                viewController?.finderAspectRatio = SBSDKAspectRatio(width: width, andHeight: height)
            }
            
            
            if let finderVerticalOffset = finderConfiguration["finderVerticalOffset"] as? CGFloat {
                let padding = finderConfiguration["finderMinimumPadding"] as? CGFloat ?? 0
                viewController?.finderMinimumInset.top = padding + finderVerticalOffset < 0 ? finderVerticalOffset : 0;
                viewController?.finderMinimumInset.bottom = padding + finderVerticalOffset < 0 ? -2*finderVerticalOffset : 0;
                viewController?.finderMinimumInset.left = padding
                viewController?.finderMinimumInset.right = padding
            }
            
        }
        
        if let scannerConfiguration: [String:Any] = configuration?["scannerConfiguration"] as? [String : Any] {
            if let acceptedDocumentType = scannerConfiguration["acceptedDocumentTypes"] as? [String : Any] {
                viewController?.acceptedDocumentTypes = SBSDKBarcodeDocumentType.fromArguments(acceptedDocumentType)
            }
            
            if let barcodeFormats = scannerConfiguration["barcodeFormats"] as? [String : Any] {
                viewController?.acceptedBarcodeTypes = SBSDKBarcodeType.fromArguments(barcodeFormats)
            }
            
            if let engineMode = scannerConfiguration["engineMode"] as? [String:Any] {
                viewController?.engineMode = SBSDKBarcodeEngineMode.fromArguments(engineMode)!
            }
            
            if let additionalParameters = scannerConfiguration["additionalParameters"] as? [String:Any] {
                viewController?.additionalDetectionParameters = SBSDKBarcodeAdditionalParameters(from:additionalParameters)
            }
            
            if let detectionIntervalInMs = scannerConfiguration["detectionIntervalInMs"] as? Int {
                viewController?.detectionRate = UInt(detectionIntervalInMs)
            }
            
            if let barcodeImageGenerationType = scannerConfiguration["barcodeImageGenerationType"] as? String {
                imageGenerationType = SBSDKBarcodeImageGenerationType(from:barcodeImageGenerationType)
                viewController?.barcodeImageGenerationType = imageGenerationType
            }
        }
    }
    
    fileprivate func toggleFlash(arguments : [String:Any?]?) {
        if let flashEnabled = arguments?["enabled"] as? Bool {
            viewController?.flashLightEnabled = flashEnabled
        }
    }
    
    func handleCameraCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let arguments = call.arguments as? [String:Any?]
        switch call.method {
        case "flashEnabled" : toggleFlash(arguments: arguments)
        case "dispose" : self.dispose()
        default: self.returnUnknownCallError(call: call, result: result)
        }
    }
    
    func handleDetectorCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "enable" : shouldDetectBarcodes = true
        case "disable" : shouldDetectBarcodes = false
        default: self.returnUnknownCallError(call: call, result: result)
        }
    }
}
