import 'package:json_annotation/json_annotation.dart';
import 'package:scanbot_sdk/barcode_scanning_data.dart';

import 'camera_configuration.dart';

part 'barcode_scanner_configuration.g.dart';

@JsonSerializable(
    createFactory: false, includeIfNull: false, explicitToJson: true)
class BarcodeCameraConfiguration extends CameraConfiguration {
  /// Controls whether the flash should be initially enabled.
  /// The default value is FALSE.
  bool? flashEnabled;

  ///Set current zoom by a linear zoom value ranging from 0f to 1.0f. Optical Zoom Level 0f represents the minimum zoom while Optical Zoom Level 1.0f represents the maximum zoom.
  ///Default value is 0f.
  double? cameraZoomFactor;

  @override
  @JsonKey(name: "detectorType")
  final DetectorType detectorType = DetectorType.barcode;

  FinderConfiguration? finder;

  BarcodeClassicScannerConfiguration scannerConfiguration;

  BarcodeCameraConfiguration({
    required this.scannerConfiguration,
    this.flashEnabled,
    this.finder,
    this.cameraZoomFactor,
  });

  @override
  Map<String, dynamic> toJson() => _$BarcodeCameraConfigurationToJson(this);
}

@JsonSerializable(
    createFactory: false, includeIfNull: false, explicitToJson: true)
class BarcodeClassicScannerConfiguration {
  @JsonKey(name: "detectorType")
  DetectorType detectorType = DetectorType.barcode;

  /// List of accepted Barcode Formats; any formats that are not included in this
  /// list will not be detected by the `BarcodeCameraView`. By default, the list
  /// includes all supported barcode types.
  List<BarcodeFormat> barcodeFormats;

  /// List of accepted Document Formats; any document formats that are not included
  /// in this list will be ignored by the `BarcodeCameraView`. By default, the list
  /// includes all supported document types.
  List<AcceptedDocumentFormats> acceptedDocumentFormats;

  /// The checksum algorithm for MSI Plessey barcodes.
  /// The default value is `Mod10'
  MSIPlesseyChecksumAlgorithm? msiPlesseyChecksumAlgorithm;

  /// The engine mode of the barcode recognizer. Defaults to NEXT_GEN.
  /// To use legacy recognizer, please specify LEGACY
  EngineMode? engineMode;

  /// Additional parameters for tweaking the detection of barcodes.
  BarcodeAdditionalParameters? additionalParameters;

  /// delay betwen sucessfull scanning of the barcode and next scanning
  int detectionIntervalInMs;

  /// Param that handle whether we need to save image from camera preview or make a snapshot while snapping barcode
  /// image uri will be returned in [BarcodeScanningResult.snappedImage] property
  BarcodeImageGenerationType barcodeImageGenerationType =
      BarcodeImageGenerationType.NONE;

  /// Changes sensitivity of auto-snapping. That is: the more sensitive it is the faster it shoots.
  /// This parameter only works when `barcodeImageGenerationType = BarcodeImageGenerationType.CAPTURED_IMAGE`
  /// Sensitivity must be within [0..1] range. A value of 1.0 triggers automatic capturing immediately,
  /// a value of 0.0 delays the automatic capturing by 3 seconds.
  ///
  /// The default value is 0.66 (1 sec) so you will get snapped barcode result with saved image each not offen than 1 second.
  ///
  /// @param sensitivity only accepts values within [0..1] range.
  double autoSnappingSensitivity;

  BarcodeClassicScannerConfiguration(
      {this.barcodeFormats = const [],
      this.acceptedDocumentFormats = const [],
      this.msiPlesseyChecksumAlgorithm,
      this.engineMode,
      this.additionalParameters,
      this.detectionIntervalInMs = 1000,
      this.barcodeImageGenerationType = BarcodeImageGenerationType.NONE,
      this.autoSnappingSensitivity = 0.66})
      : assert(autoSnappingSensitivity >= 0 && autoSnappingSensitivity <= 1,
            "autoSnappingSensitivity should be in range between 0 to 1");

  Map<String, dynamic> toJson() =>
      _$BarcodeClassicScannerConfigurationToJson(this);
}
