import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:scanbot_sdk/common_data.dart' as c;

import 'camera_configuration.dart';
import 'live_detection.dart';

class ScanbotCamera extends StatefulWidget {
  final CameraConfiguration configuration;

  final LiveDetector? detector;

  /// triggers when camera plugin make long work eg. saving snap to the image etc.
  final Function(bool)? onHeavyOperationProcessing;

  /// returns page after sucessfull snap
  final Function(c.Page)? onPictureTaken;

  final Function(ScanbotCameraController) onWidgetReady;

  const ScanbotCamera(
      {Key? key,
      required this.configuration,
      required this.onWidgetReady,
      this.detector,
      this.onHeavyOperationProcessing,
      this.onPictureTaken})
      : super(key: key);

  @override
  _ScanbotCameraWidgetState createState() => _ScanbotCameraWidgetState();
}

class _ScanbotCameraWidgetState extends State<ScanbotCamera> {
  ScanbotCameraController? _scanbotCameraController;

  @override
  Widget build(BuildContext context) {
    var configuration = widget.configuration;
    if (defaultTargetPlatform == TargetPlatform.android) {
      return AndroidView(
        viewType: "scanbot_sdk_camera",
        onPlatformViewCreated: _onPlatformViewCreated,
        creationParams: configuration.toJson(),
        creationParamsCodec: const StandardMessageCodec(),
      );
    }
    if (defaultTargetPlatform == TargetPlatform.iOS) {
      return UiKitView(
        viewType: "scanbot_sdk_camera",
        layoutDirection: TextDirection.ltr,
        onPlatformViewCreated: _onPlatformViewCreated,
        creationParams: configuration.toJson(),
        creationParamsCodec: const StandardMessageCodec(),
      );
    }
    return Text(
        '$defaultTargetPlatform is not yet supported by the camera plugin');
  }

  void _onPlatformViewCreated(int id) {
    var scanbotCameraController = ScanbotCameraController(
        viewId: id,
        detector: widget.detector,
        onHeavyOperationProcessing: widget.onHeavyOperationProcessing,
        onPictureTaken: widget.onPictureTaken);
    _scanbotCameraController = scanbotCameraController;
    widget.onWidgetReady.call(scanbotCameraController);
  }

  @override
  void dispose() {
    super.dispose();
    _scanbotCameraController?.disposeCamera();
  }
}

class ScanbotCameraController {
  int viewId;

  ScanbotCameraController({
    required this.viewId,
    this.onPictureTaken,
    this.onHeavyOperationProcessing,
    this.detector,
  }) {
    cameraChannel = const MethodChannel('scanbot_sdk_camera');
    cameraChannel.setMethodCallHandler(_resultHandler);
  }

  late MethodChannel cameraChannel;

  LiveDetector? detector;

  /// triggers when camera plugin make long work eg. saving snap to the image etc.
  final Function(bool)? onHeavyOperationProcessing;

  /// returns page after sucessfull snap
  final Function(c.Page)? onPictureTaken;

  // todo copy to document detector
  // void takeSnapshot() async {
  //   await cameraChannel.invokeListMethod("takeSnapshot");
  // }

  void _onNativeCameraReady() {
    if (detector != null) {
      _attachDetector(detector!);
    }
  }

  void _attachDetector(LiveDetector detector) {
    this.detector?.attach(this);
  }

  void pauseCamera() async {
    await cameraChannel.invokeListMethod("pause");
  }
  // temporarily disabled
  // void toggleAutoFocus() async {
  //   await cameraChannel.invokeListMethod("toggleAutoFocus");
  // }

  void toggleContinuosFocus() async {
    await cameraChannel.invokeListMethod("toggleContinuousFocus");
  }

  void resumeCamera() async {
    await cameraChannel.invokeListMethod("resume");
  }

  void disposeCamera() async {
    await cameraChannel.invokeListMethod("dispose");
  }

  void setFlashEnabled(bool ebabled) async {
    await cameraChannel.invokeListMethod("flashEnabled", {"enabled": ebabled});
  }

  Future<dynamic> _resultHandler(MethodCall methodCall) async {
    switch (methodCall.method) {
      case 'onPictureTaken':
        c.Page page = c.Page.fromJson(jsonDecode(methodCall.arguments));
        onPictureTaken?.call(page);
        break;
      case 'onCameraOpen':
        _onNativeCameraReady();
        break;
      case 'onShowHeavyProcessing':
        onHeavyOperationProcessing?.call(methodCall.arguments as bool);
        break;
      default:
    }
    return;
  }
}
